package com.vaadin.addon.calendar.demo;

import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import com.vaadin.addon.calendar.event.BasicEvent;
import com.vaadin.addon.calendar.event.CalendarEvent;
import com.vaadin.addon.calendar.event.CalendarEventEditor;
import com.vaadin.addon.calendar.event.CalendarEventProvider;
import com.vaadin.addon.calendar.gwt.client.ui.VCalendar;
import com.vaadin.addon.calendar.ui.Calendar;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.BackwardEvent;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.DateClickEvent;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.EventClick;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.EventClickHandler;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.ForwardEvent;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.MoveEvent;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.RangeSelectEvent;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.RangeSelectHandler;
import com.vaadin.addon.calendar.ui.CalendarComponentEvents.WeekClick;
import com.vaadin.addon.calendar.ui.handler.BasicBackwardHandler;
import com.vaadin.addon.calendar.ui.handler.BasicDateClickHandler;
import com.vaadin.addon.calendar.ui.handler.BasicEventMoveHandler;
import com.vaadin.addon.calendar.ui.handler.BasicEventResizeHandler;
import com.vaadin.addon.calendar.ui.handler.BasicForwardHandler;
import com.vaadin.addon.calendar.ui.handler.BasicWeekClickHandler;
import com.vaadin.annotations.PreserveOnRefresh;
import com.vaadin.data.Item;
import com.vaadin.data.util.BeanItem;
import com.vaadin.server.Page;
import com.vaadin.server.VaadinRequest;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Component;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.FormFieldFactory;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

@SuppressWarnings({ "deprecation", "serial" })
@PreserveOnRefresh
public class CustomizedCalendarDemo extends UI {

    private MyEventProvider provider = new MyEventProvider();

    @Override
    protected void init(VaadinRequest request) {
        setLocale(Locale.US);

        final Calendar cal = new Calendar(provider);
        cal.setSizeFull();

        cal.setStartDate(new Date());
        cal.setEndDate(new Date());

        // Make sure the date is in the same year as today
        cal.setHandler(new BasicBackwardHandler() {

            @Override
            protected void setDates(BackwardEvent event, Date start, Date end) {
                java.util.Calendar calendar = event.getComponent().getInternalCalendar();
                if (isThisYear(calendar, end) && isThisYear(calendar, start)) {
                    super.setDates(event, start, end);
                }
            }
        });

        // Make sure the date is in the same year as today
        cal.setHandler(new BasicForwardHandler() {

            @Override
            protected void setDates(ForwardEvent event, Date start, Date end) {
                java.util.Calendar calendar = event.getComponent().getInternalCalendar();
                if (isThisYear(calendar, start) && isThisYear(calendar, end)) {
                    super.setDates(event, start, end);
                }
            }
        });

        // Set a date click handler
        cal.setHandler(new BasicDateClickHandler() {

            @Override
            public void dateClick(DateClickEvent event) {
                Calendar cal = event.getComponent();
                long currentCalDateRange = cal.getEndDate().getTime() - cal.getStartDate().getTime();

                if (currentCalDateRange < VCalendar.DAYINMILLIS) {
                    // Change the date range to the current week
                    cal.setStartDate(cal.getFirstDateForWeek(event.getDate()));
                    cal.setEndDate(cal.getLastDateForWeek(event.getDate()));

                } else {
                    // Default behaviour, change date range to one day
                    super.dateClick(event);
                }
            }
        });

        // allow moving to week view by clicking the weeknumber only if the
        // weeks start and end dates are on the current month
        cal.setHandler(new BasicWeekClickHandler() {

            @Override
            protected void setDates(WeekClick event, Date start, Date end) {
                java.util.Calendar calendar = event.getComponent().getInternalCalendar();
                if (isThisMonth(calendar, start) && isThisMonth(calendar, end)) {
                    super.setDates(event, start, end);
                }
            }
        });

        cal.setHandler(new EventClickHandler() {

            public void eventClick(EventClick event) {
                BasicEvent e = (BasicEvent) event.getCalendarEvent();
                new Notification("Event clicked: " + e.getCaption(), e.getDescription()).show(Page.getCurrent());
            }
        });

        // Set the event move listener
        cal.setHandler(new BasicEventMoveHandler() {

            private java.util.Calendar javaCalendar;

            @Override
            public void eventMove(MoveEvent event) {
                javaCalendar = event.getComponent().getInternalCalendar();
                super.eventMove(event);

            }

            @Override
            protected void setDates(CalendarEventEditor event, Date start, Date end) {
                if (isThisMonth(javaCalendar, start) && isThisMonth(javaCalendar, end)) {
                    super.setDates(event, start, end);
                }
            }
        });

        // Set the drag selection handler
        cal.setHandler(new RangeSelectHandler() {

            public void rangeSelect(RangeSelectEvent event) {
                BasicEvent calendarEvent = new BasicEvent();
                calendarEvent.setStart(event.getStart());
                calendarEvent.setEnd(event.getEnd());

                // Create popup window and add a form in it.
                VerticalLayout layout = new VerticalLayout();
                layout.setMargin(true);
                layout.setSpacing(true);

                final Window w = new Window(null, layout);
                w.setWidth("400px");
                w.setModal(true);
                w.center();

                // Wrap the calendar event to a BeanItem and pass it to the form
                final BeanItem<CalendarEvent> item = new BeanItem<CalendarEvent>(calendarEvent);

                final Form form = new Form();
                //form.setWriteThrough(false);
                form.setItemDataSource(item);
                form.setFormFieldFactory(new FormFieldFactory() {

                    public Field<?> createField(Item item, Object propertyId, Component uiContext) {
                        if (propertyId.equals("caption")) {
                            TextField f = new TextField("Caption");
                            f.setNullRepresentation("");
                            f.focus();
                            return f;

                        }
                        return null;
                    }
                });
                form.setVisibleItemProperties(new Object[] { "caption" });

                layout.addComponent(form);

                HorizontalLayout buttons = new HorizontalLayout();
                buttons.setSpacing(true);
                buttons.addComponent(new Button("OK", new ClickListener() {

                    public void buttonClick(ClickEvent event) {
                        form.commit();
                        // Update event provider's data source
                        provider.addEvent(item.getBean());
                        // Calendar needs to be repainted
                        cal.markAsDirty();

                        UI.getCurrent().removeWindow(w);
                    }
                }));
                buttons.addComponent(new Button("Cancel", new ClickListener() {

                    public void buttonClick(ClickEvent event) {
                        UI.getCurrent().removeWindow(w);
                    }
                }));
                layout.addComponent(buttons);
                layout.setComponentAlignment(buttons, Alignment.BOTTOM_RIGHT);

                UI.getCurrent().addWindow(w);
            }
        });

        cal.setHandler(new BasicEventResizeHandler() {

            private static final long twelveHoursInMs = 12 * 60 * 60 * 1000;

            @Override
            protected void setDates(CalendarEventEditor event, Date start, Date end) {
                long eventLength = end.getTime() - start.getTime();
                if (eventLength <= twelveHoursInMs) {
                    super.setDates(event, start, end);
                }
            }
        });

        Button monthViewButton = new Button("Show month");
        monthViewButton.addClickListener(new ClickListener() {

            public void buttonClick(ClickEvent event) {
                GregorianCalendar gc = new GregorianCalendar(cal.getTimeZone(), cal.getLocale());
                gc.setTime(cal.getStartDate());

                long currentCalDateRange = cal.getEndDate().getTime() - cal.getStartDate().getTime();
                if (currentCalDateRange > (VCalendar.DAYINMILLIS * 7)) {
                    return;
                }

                // Reset calendar's start time to the target month's first day.
                gc.set(GregorianCalendar.DATE, gc.getMinimum(GregorianCalendar.DATE));
                gc.set(GregorianCalendar.HOUR_OF_DAY, 0);
                gc.set(GregorianCalendar.MINUTE, 0);
                gc.set(GregorianCalendar.SECOND, 0);
                gc.set(GregorianCalendar.MILLISECOND, 0);
                cal.setStartDate(gc.getTime());
                gc.add(GregorianCalendar.MONTH, 1);
                gc.add(GregorianCalendar.DATE, -1);
                cal.setEndDate(gc.getTime());

            }
        });

        VerticalLayout layout = new VerticalLayout();
        layout.setSizeFull();
        layout.setMargin(true);
        layout.addComponent(monthViewButton);
        layout.addComponent(cal);
        layout.setExpandRatio(cal, 1);
        setContent(layout);
        setSizeFull();
    }

    public static boolean isThisYear(java.util.Calendar calendar, Date date) {
        calendar.setTime(new Date());
        int thisYear = calendar.get(java.util.Calendar.YEAR);
        calendar.setTime(date);
        return calendar.get(java.util.Calendar.YEAR) == thisYear;
    }

    public static boolean isThisMonth(java.util.Calendar calendar, Date date) {
        calendar.setTime(new Date());
        int thisMonth = calendar.get(java.util.Calendar.MONTH);
        calendar.setTime(date);
        return calendar.get(java.util.Calendar.MONTH) == thisMonth;
    }

    public static class MyEventProvider implements CalendarEventProvider {

        private List<CalendarEvent> events = new ArrayList<CalendarEvent>();

        public MyEventProvider() {
            events = new ArrayList<CalendarEvent>();
            GregorianCalendar cal = new GregorianCalendar();
            cal.setTime(new Date());

            Date start = cal.getTime();
            cal.add(GregorianCalendar.HOUR, 5);
            Date end = cal.getTime();
            BasicEvent event = new BasicEvent();
            event.setCaption("My Event");
            event.setDescription("My Event Description");
            event.setStart(start);
            event.setEnd(end);
            events.add(event);
        }

        public void addEvent(CalendarEvent BasicEvent) {
            events.add(BasicEvent);
        }

        public List<CalendarEvent> getEvents(Date startDate, Date endDate) {
            return events;
        }
    }

    public static BasicEvent getEvent(String caption, Date start, Date end) {
        BasicEvent event = new BasicEvent();
        event.setCaption(caption);
        event.setStart(start);
        event.setEnd(end);

        return event;
    }
}
